@@ -26,8 +26,6 @@ from pyrf.connectors.twisted_async import TwistedConnector
 from pyrf.config import TriggerSettings, TRIGGER_TYPE_LEVEL
 from pyrf.units import M
 from pyrf.devices.thinkrf import WSA
-from pyrf.vrt import (I_ONLY, VRT_IFDATA_I14Q14, VRT_IFDATA_I14,
-    VRT_IFDATA_I24, VRT_IFDATA_PSD8)
 
 from pyrf.gui.util import find_nearest_index
 from pyrf.gui.plot_widget import Plot
@@ -36,10 +34,11 @@ from pyrf.gui.frequency_controls import FrequencyControls
 from pyrf.gui.amplitude_controls import AmplitudeControls
 from pyrf.gui.discovery_widget import DiscoveryWidget
 from pyrf.gui.trace_controls import TraceControls
+from pyrf.gui.measurements_widget import MeasurementControls
 
 VIEW_OPTIONS = [
     ('&IQ Plots', 'iq_plots', False),
-    ('&Spectrogram Plot', 'waterfall_plot', False),
+    ('&Spectrogram', 'waterfall_plot', False),
     ('&Persistence Plot', 'persistence_plot', False),
     ]
 
@@ -53,14 +52,6 @@ DEVELOPER_OPTIONS = [
     ('Apply &Hanning Window', 'dsp.apply_window', True),
     ]
 
-# FIXME: we shouldn't be calculating fft in this module
-ZIF_BITS = 2**13
-CONST_POINTS = 512
-
-# FIXME: calculate from device properties instead
-IQ_PLOT_YMIN = {'ZIF': -1, 'HDR': -1, 'SH': -1, 'SHN': -1, 'IQIN': -1, 'DD': -1}
-IQ_PLOT_YMAX = {'ZIF': 1, 'HDR': 1, 'SH': -1, 'SHN': -1, 'IQIN': 1, 'DD': 1}
-
 MINIMUM_WIDTH = 600
 MINIMUM_HEIGHT = 600
 
@@ -111,6 +102,11 @@ class MainWindow(QtGui.QMainWindow):
         self.stop_action = QtGui.QAction('&Stop Recording', self)
         self.stop_action.triggered.connect(self.stop_recording)
         self.stop_action.setDisabled(True)
+        self.start_csv_export = QtGui.QAction('&Start Exporting CSV', self)
+        self.start_csv_export.triggered.connect(self.start_csv)
+        self.stop_csv_export = QtGui.QAction('&Stop Exporting CSV', self)
+        self.stop_csv_export.triggered.connect(self.stop_csv)
+        self.stop_csv_export.setDisabled(True)
         exit_action = QtGui.QAction('&Exit', self)
         exit_action.setShortcut('Ctrl+Q')
         exit_action.triggered.connect(self.close)
@@ -121,6 +117,8 @@ class MainWindow(QtGui.QMainWindow):
         file_menu.addSeparator()
         file_menu.addAction(self.record_action)
         file_menu.addAction(self.stop_action)
+        file_menu.addAction(self.start_csv_export)
+        file_menu.addAction(self.stop_csv_export)
         file_menu.addSeparator()
         file_menu.addAction(exit_action)
 
@@ -183,6 +181,30 @@ class MainWindow(QtGui.QMainWindow):
         if playback_filename:
             self.start_playback(playback_filename)
 
+    def start_csv(self):
+
+        names = glob.glob('csv-*.csv')
+        last_index = -1
+        for n in names:
+            try:
+                last_index = max(last_index, int(n[4:-4]))
+
+            except ValueError:
+                pass
+        filename = 'csv-%04d.csv' % (last_index + 1)
+        playback_filename, file_type = QtGui.QFileDialog.getSaveFileName(self,
+            "CSV File", filename, "CSV File (*.csv)")
+
+        if playback_filename:
+            self.controller.start_csv_export(playback_filename)
+            self.start_csv_export.setDisabled(True)
+            self.stop_csv_export.setDisabled(False)
+
+    def stop_csv(self):
+        self.start_csv_export.setDisabled(False)
+        self.stop_csv_export.setDisabled(True)
+        self.controller.stop_csv_export()
+
     def start_playback(self, playback_filename):
         self.record_action.setDisabled(True)
         self.stop_action.setDisabled(True)
@@ -230,6 +252,7 @@ class MainPanel(QtGui.QWidget):
         self.controller = controller
         controller.device_change.connect(self.device_changed)
         controller.state_change.connect(self.state_changed)
+        controller.plot_change.connect(self.plot_changed)
         controller.capture_receive.connect(self.capture_received)
         controller.options_change.connect(self.options_changed)
 
@@ -283,9 +306,8 @@ class MainPanel(QtGui.QWidget):
                 self._plot.center_view(freq,
                                        full_bw,
                                        self._amplitude_group.get_min_level(),
-                                       self._amplitude_group.get_ref_level())
-                self._plot.iq_window.setYRange(IQ_PLOT_YMIN[rfe_mode],
-                                               IQ_PLOT_YMAX[rfe_mode])
+                                       self._amplitude_group.get_max_level())
+                self._plot.center_iq_plots()
             else:
                 freq = state.center
                 full_bw = state.span
@@ -293,9 +315,8 @@ class MainPanel(QtGui.QWidget):
                 self._plot.center_view(freq - full_bw/2,
                                         freq + full_bw/2,
                                        self._amplitude_group.get_min_level(),
-                                       self._amplitude_group.get_ref_level())
-                self._plot.iq_window.setYRange(IQ_PLOT_YMIN[rfe_mode],
-                                        IQ_PLOT_YMAX[rfe_mode])
+                                       self._amplitude_group.get_max_level())
+                self._plot.center_iq_plots()
         if 'device_settings.iq_output_path' in changed:
             if state.device_settings['iq_output_path'] == 'CONNECTOR':
                 # remove plots
@@ -333,6 +354,9 @@ class MainPanel(QtGui.QWidget):
         if 'span' in changed:
             self.update_span_label()
 
+    def plot_changed(self, state, changed):
+        self.plot_state = state
+
     def show_labels(self):
         self._rbw_label.show()
         self._span_label.show()
@@ -340,6 +364,8 @@ class MainPanel(QtGui.QWidget):
         self._mask_label.show()
         for m in self.marker_labels:
             m.show()
+        for c in self.channel_power_labels:
+            c.show()
 
     def hide_labels(self):
         self._rbw_label.hide()
@@ -348,6 +374,8 @@ class MainPanel(QtGui.QWidget):
         self._mask_label.hide()
         for m in self.marker_labels:
             m.hide()
+        for c in self.channel_power_labels:
+            c.hide()
 
     def update_rbw_label(self):
         rfe_mode = self.gui_state.rfe_mode()
@@ -376,7 +404,7 @@ class MainPanel(QtGui.QWidget):
 
         self.marker_labels = []
         marker_label, delta_label, diff_label, rbw_label, span_label = self._marker_labels()
-
+        channel_power_labels = self._channel_power_labels()
         grid.addWidget(self._mask_label, 0, 0, 2, self.plot_width)
         grid.addWidget(marker_label, 0, 3, 1, 2)
         grid.addWidget(delta_label, 0, 5, 1, 2)
@@ -384,11 +412,16 @@ class MainPanel(QtGui.QWidget):
         grid.addWidget(self._rbw_label, 0, 0, 1, 2)
         grid.addWidget(self._span_label, 0, 9, 1, 2)
         grid.addWidget(self._plot_layout(), 1, 0, 14, self.plot_width)
+        x = 2
+        for label in channel_power_labels:
+            grid.addWidget(label, 1, x, 1, 2)
+            x += 3
         y = 0
         x = self.plot_width
         controls_layout = QtGui.QVBoxLayout()
 
         controls_layout.addWidget(self._freq_controls())
+        controls_layout.addWidget(self._measurement_controls())
         controls_layout.addWidget(self._amplitude_controls())
         controls_layout.addWidget(self._device_controls())
         controls_layout.addWidget(self._trace_controls())
@@ -428,7 +461,7 @@ class MainPanel(QtGui.QWidget):
         self._freq_group = FrequencyControls(self.controller)
         self.control_widgets.append(self._freq_group)
         return self._freq_group
-    
+
     def _amplitude_controls(self):
         self._amplitude_group = AmplitudeControls(self.controller, self._plot)
         self.control_widgets.append(self._amplitude_group)
@@ -439,6 +472,11 @@ class MainPanel(QtGui.QWidget):
         self.control_widgets.append(self.trace_group)
         return self.trace_group
 
+    def _measurement_controls(self):
+        self.measure_group = MeasurementControls(self.controller)
+        self.control_widgets.append(self.measure_group)
+        return self.measure_group
+
     def _dsp_controls(self):
         self._dsp_group = DSPWidget()
         self.control_widgets.append(self._dsp_group)
@@ -450,23 +488,29 @@ class MainPanel(QtGui.QWidget):
         return self._dev_group
 
     def _marker_labels(self):
+        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Fixed)
 
         marker_label = QtGui.QLabel('')
         marker_label.setAlignment(QtCore.Qt.AlignLeft)
+        marker_label.setSizePolicy(sizePolicy)
 
         delta_label = QtGui.QLabel('')
         delta_label.setAlignment(QtCore.Qt.AlignLeft)
+        delta_label.setSizePolicy(sizePolicy)
 
         span_label = QtGui.QLabel('')
         span_label.setStyleSheet(fonts.MARKER_LABEL_FONT % (colors.BLACK_NUM + colors.GREY_NUM))
         span_label.setAlignment(QtCore.Qt.AlignLeft)
+        span_label.setSizePolicy(sizePolicy)
 
         rbw_label = QtGui.QLabel('')
         rbw_label.setStyleSheet(fonts.MARKER_LABEL_FONT % (colors.BLACK_NUM + colors.GREY_NUM))
         rbw_label.setAlignment(QtCore.Qt.AlignRight)
+        rbw_label.setSizePolicy(sizePolicy)
 
         diff_label = QtGui.QLabel('')
         diff_label.setAlignment(QtCore.Qt.AlignLeft)
+        diff_label.setSizePolicy(sizePolicy)
         self._diff_label = diff_label
         self._rbw_label = rbw_label
         self._span_label = span_label
@@ -475,6 +519,18 @@ class MainPanel(QtGui.QWidget):
         self.marker_labels.append(delta_label)
         return marker_label,delta_label, diff_label, rbw_label, span_label
 
+    def _channel_power_labels(self):
+        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Fixed)
+
+        self.channel_power_labels = []
+
+        for color in colors.TRACE_COLORS:
+            label = QtGui.QLabel('')
+            label.setSizePolicy(sizePolicy)
+            label.setStyleSheet(fonts.MARKER_LABEL_FONT % (colors.BLACK_NUM + color))
+            self.channel_power_labels.append(label)
+        return self.channel_power_labels
+
     def capture_received(self, state, fstart, fstop, raw, power, usable, segments):
         """
         :param state: SpecAState when capture was requested
@@ -496,12 +552,13 @@ class MainPanel(QtGui.QWidget):
         self.update_trace()
         self.update_marker()
         self.update_diff()
+        self.update_channel_power()
         if (not self.controller.applying_user_xrange() and
                 not self.controller.get_options()['free_plot_adjustment']):
             self._plot.center_view(fstart,
                                    fstop,
                                    self._amplitude_group.get_min_level(),
-                                   self._amplitude_group.get_ref_level())
+                                   self._amplitude_group.get_max_level())
 
         if self.iq_plots_enabled:
             self.update_iq()
@@ -560,42 +617,7 @@ class MainPanel(QtGui.QWidget):
 
         if not self.raw_data:
                 return
-        trace = self._plot.traces[0]
-
-        if not (trace.write or trace.max_hold or trace.min_hold or trace.store):
-            return
-        if not trace.store:
-            data_pkt = self.raw_data
-            trace.raw_packet = self.raw_data
-        else:
-            data_pkt = trace.raw_packet
-
-        if data_pkt.stream_id == VRT_IFDATA_I14Q14:
-            data = data_pkt.data.numpy_array()
-            i_data = np.array(data[:,0], dtype=float)/ZIF_BITS
-            q_data = np.array(data[:,1], dtype=float)/ZIF_BITS
-            self._plot.i_curve.setData(i_data)
-            self._plot.q_curve.setData(q_data)
-            self._plot.const_plot.clear()
-            self._plot.const_plot.addPoints(
-                x = i_data[0:CONST_POINTS],
-                y = q_data[0:CONST_POINTS],
-                symbol = 'o',
-                size = 1, pen = 'y',
-                brush = 'y')
-
-        else:
-            data = data_pkt.data.numpy_array()
-            i_data = np.array(data, dtype=float)
-
-            if data_pkt.stream_id == VRT_IFDATA_I14:
-                i_data = i_data /ZIF_BITS
-
-            elif data_pkt.stream_id == VRT_IFDATA_I24:
-                i_data = i_data / (np.mean(i_data)) - 1
-            self._plot.i_curve.setData(i_data)
-
-            self._plot.q_curve.clear()
+        self._plot.update_iq_plots(self.raw_data)
 
     def update_marker(self):
             num = 1
@@ -646,6 +668,16 @@ class MainPanel(QtGui.QWidget):
             self._diff_label.setText(delta_text)
         else:
             self._diff_label.hide()
+
+    def update_channel_power(self):
+
+        for label, trace in zip(self.channel_power_labels, self._plot.traces):
+            if trace.calc_channel_power and not trace.blank:
+                label.setStyleSheet(fonts.MARKER_LABEL_FONT % (colors.BLACK_NUM + trace.color))
+                label.setText(("Channel Power: %0.2f dBm" % trace.channel_power))
+            else:
+                label.setText('')
+
     def enable_controls(self):
         for item in self.control_widgets:
             item.setEnabled(True)
